# For test

from flask import Flask
from flask_mongoengine import MongoEngine
from mongoengine import fields
from flask_restful import Resource, Api
from flask_rest_mongo.mix import *
import mongoengine

class Config(object):

    ENV = 'development'
    SERVER_NAME = '127.0.0.1:7788'
    DEBUG = True
    MONGODB_SETTINGS = {
        'db': 'flaskTest',
        'host': '127.0.0.1',
        'port': 27017
    }

if __name__ == "__main__":
    app = Flask(__name__)
    db = MongoEngine()
    api = Api()
    app.config.from_object(Config())
    

    class UserTest(db.DynamicDocument):
        username = fields.StringField(primary_key=True)
        email = fields.EmailField()
        password = fields.StringField()
        def to_json(self):
            return {
                'username':self.username,
                'email':self.email
            }

    class DataTest(db.DynamicDocument):
        data = fields.DictField()
        list_data = fields.ListField()
        def to_json(self, *args, **kwargs):
                return {
                    'id':str(self.id),
                    'data':self.data,
                    'list':self.list_data
                }

    class TestDocument(db.DynamicDocument):
        user = fields.ReferenceField(UserTest, reverse_delete_rule=mongoengine.CASCADE, required=True)
        data_list = fields.ListField(fields.ReferenceField(DataTest))
        count = fields.IntField()
        factor = fields.FloatField()
        
    
    class UserListCreate(Resource, MixListCreate):
        document = UserTest
        exclude = ['password']
        page = True
    api.add_resource(UserListCreate, '/user/')
    class UserDetail(Resource, MixRetrialUpdateDelete):
        document = UserTest
        exclude = ['password']
    api.add_resource(UserDetail, '/user/detail/')

    class DataListCreate(Resource, MixListCreate):
        document = DataTest
        page = True
    api.add_resource(DataListCreate, '/data/')
    class DataDetail(Resource, MixRetrialUpdateDelete):
        document = DataTest
    api.add_resource(DataDetail, '/data/detail/')

    class TestDocumentListCreate(Resource, MixListCreate):
        document = TestDocument
        page = True
    api.add_resource(TestDocumentListCreate, '/test/')
    class TestDocumentDetail(Resource, MixRetrialUpdateDelete):
        document = TestDocument
    api.add_resource(TestDocumentDetail, '/test/detail/')


    db.init_app(app)
    api.init_app(app)

    app.run()



import json

from flask import request
from flask_restful import abort, reqparse

from .fields import parser_field
from .filters import field_filter


class Mixs(object):
    """
    docstring
    """
    document = None
    permissions = {}
    page = None

    def __init__(self):
        self._parser = None
        self._arg_map = {}
        self._page_parser = None
        self._filter_parser = None
        self._init_parser()

    @property
    def _fields(self):
        return self.document._fields

    def get_document(self):

        return self.document

    @property
    def _id(self):
        return self.document._meta['id_field']

    def get_object(self):
        args = self._valid_args(self._parser)
        if self._id not in args:
            abort(400, msg='%s is required'%self._id, error=1)
        obj = self.document.objects(**{self._id: args[self._id]}).first()
        if obj is None:
            abort(404, msg='cound not find object with %s=%s'%(self._id, args[self._id]), error=1)
        return obj

    def _valid_args(self, parser):
        args = parser.parse_args()
        print(args)
        def f(k): return k[1] is not None
        return dict(filter(f, args.items()))

    def _check_permission(self, name: str, *args):
        if isinstance(self.permissions, dict):
            func = self.permissions.get(name.lower(), None)
            if func is not None:
                if isinstance(func, list):
                    for f in func:
                        f(*args)
            return
        if isinstance(self.permissions, list):
            for f in self.permissions:
                f(*args)
            return
        if self.permissions is not None:
            self.permissions(*args)

    def _init_parser(self):
        self._init_fields_parser()
        self._init_filter_parser()
        self._init_page_parser()

    def _init_fields_parser(self):
        self._parser = reqparse.RequestParser()
        fields = self._fields
        id = self._id
        self._parser.add_argument(
                parser_field(name='id', field=fields[id], dest=id))
        del fields[id]
        for key in fields:
            self._parser.add_argument(
                parser_field(name=key, field=fields[key]))

    def _init_filter_parser(self):
        pass

    def _init_page_parser(self):
        pass

    def _obj_to_json_dict(self, obj):
        s = obj.to_json()
        if isinstance(s, str):
            return json.loads(s)
        return s


class MixList(Mixs):
    """
    docstring
    """
    page = True
    page_key = 'page'
    page_size_key = 'page_size'
    page_size_default = 10
    order_key = None
    # order_default = 'id'
    exclude = []
    max_count = 0

    @property
    def _filter_maps(self):
        pass

    def _init_page_parser(self):
        self._page_parser = reqparse.RequestParser()
        self._page_parser.add_argument(self.page_key, type=int, default=0)
        self._page_parser.add_argument(
            self.page_size_key, type=int, default=self.page_size_default)

    def _init_filter_parser(self):
        self._filter_parser = reqparse.RequestParser()
        fields = self._fields
        for key in fields:
            for aug in field_filter(key, fields[key]):
                self._filter_parser.add_argument(aug)

    def get_filter(self):
        pass

    def _get_filters(self):
        filters = self._valid_args(self._filter_parser)
        filter_rq = {}
        for key in filters:
            filter_rq[key] = filters[key]
        user_filter_parser = self.get_filter()
        if user_filter_parser is None:
            pass
        else:
            if isinstance(user_filter_parser, dict):
                user_filters = dict(user_filter_parser)
            else:
                user_filters = self._valid_args(user_filter_parser)
            for key in user_filters:
                filter_rq[key] = user_filters[key]
        return filter_rq

    def get(self):
        self._check_permission('get')
        filters = self._get_filters()
        document = self.get_document()
        docs = document.objects(**filters).exclude(*self.exclude)
        if self.order_key is not None:
            order_by = self.order_key
            if isinstance(self.order_key, str):
                order_by = (order_by,)
            docs = docs.order_by(*order_by)
        if self.page:
            pages = self._page_parser.parse_args()
            page = pages[self.page_key]
            size = pages[self.page_size_key]
            total = docs.count()
            start = page * size
            end = page * size + size
            if start >= total:
                return {'total': total, 'data': [], self.page_key: page, self.page_size_key: size}
            if end >= total:
                end = total
            docs = docs[start:end]

            data_list = []
            for obj in docs:
                data_list.append(self._obj_to_json_dict(obj))

            return {'total': total, 'data': data_list, self.page_key: page, self.page_size_key: size}
        if self.max_count > 0:
            if docs.count() > self.max_count:
                docs = docs[:self.max_count]
        data_list = []
        for obj in docs:
            data_list.append(self._obj_to_json_dict(obj))

        return {'data': data_list}


class MixCreate(Mixs):
    """
    docstring
    """
    use_json = False

    def create_obj(self):
        if self.use_json:
            data = request.json
            obj = self.document.from_json(data)
            obj.save()
        else:
            try:
                data = self._valid_args(self._parser)
                obj = self.document(**data)
                obj.save()
            except Exception as e:
                abort(400, msg=str(e), error=1)
        return obj

    def post(self):
        self._check_permission('post')
        obj = self.create_obj()
        return self._obj_to_json_dict(obj)


class MixListCreate(MixCreate, MixList):
    """
    docstring
    """
    pass


class MixRetrial(Mixs):
    """
    docstring
    """

    def get(self):
        self._check_permission('get')
        obj = self.get_object()
        return self._obj_to_json_dict(obj)


class MixUpdate(Mixs):
    """
    docstring
    """

    def put(self):
        self._check_permission('put')
        obj = self.get_object()
        update_data = self._valid_args(self._parser)
        obj.update(**update_data)
        obj.save()
        return self._obj_to_json_dict(obj)

    def patch(self):
        self._check_permission('patch')
        return self.put()


class MixDelete(Mixs):
    """
    docstring
    """

    def delete(self):
        self._check_permission('delete')
        obj = self.get_object()
        obj.delete()
        return {'msg': 'ok'}


class MixRetrialUpdate(MixRetrial, MixUpdate):
    pass


class MixRetrialDelete(MixRetrial, MixDelete):
    pass


class MixUpdateDelete(MixUpdate, MixDelete):
    pass


class MixRetrialUpdateDelete(MixRetrial, MixUpdate, MixDelete):
    pass




from flask_restful import reqparse
from mongoengine.fields import *

from .fields import parser_field

NUMBER_FILTER = [
    'ne',
    'lt' ,
    'lte',
    'gt',
    'gte'
]

STRING_FILTER = [
    'exact',
    'iexact' ,
    'contains' ,
    'icontains' ,
    'startswith' ,
    'istartswith' ,
    'endswith' ,
    'iendswith' ,
    'match'
]

ARRAY_FILTER = [
    'in',
    'nin',
    'all',
]

ARRAY_NUMBER_FILTER = [
    'size'
]

def field_filter(name, field):
    result = []
    if isinstance(field, (StringField, EmailField)):
        for key in STRING_FILTER:
            result.append(parser_field(name+'__'+key, field))
        return result
    if isinstance(field, (IntField, LongField, FloatField, DecimalField)):
        for key in NUMBER_FILTER:
            result.append(parser_field(name+'__'+key, FloatField()))
        return result
    if isinstance(field, ListField):
        for key in ARRAY_FILTER:
            result.append(parser_field(name+'__'+key, field))
        for key in ARRAY_NUMBER_FILTER:
            result.append(parser_field(name+'__'+key, IntField))
        return result
    if isinstance(field, ReferenceField):
        result.append(parser_field(name, field))
        for key in ARRAY_FILTER:
            result.append(parser_field(name+'__'+key, field, action='append'))
    return []



from mongoengine.fields import *
from bson.objectid import ObjectId
from flask_restful.reqparse import Argument
import json
import datetime
ALL_MONGO_FIELDS = [
    "StringField",
    "URLField",
    "EmailField",
    "IntField",
    "LongField",
    "FloatField",
    "DecimalField",
    "BooleanField",
    "DateTimeField",
    "DateField",
    "ComplexDateTimeField",
    "EmbeddedDocumentField",
    "ObjectIdField",
    "GenericEmbeddedDocumentField",
    "DynamicField",
    "ListField",
    "SortedListField",
    "EmbeddedDocumentListField",
    "DictField",
    "MapField",
    "ReferenceField",
    "CachedReferenceField",
    "LazyReferenceField",
    "GenericLazyReferenceField",
    "GenericReferenceField",
    "BinaryField",
    "GridFSError",
    "GridFSProxy",
    "FileField",
    "ImageGridFsProxy",
    "ImproperlyConfigured",
    "ImageField",
    "GeoPointField",
    "PointField",
    "LineStringField",
    "PolygonField",
    "SequenceField",
    "UUIDField",
    "MultiPointField",
    "MultiLineStringField",
    "MultiPolygonField",
    "GeoJsonBaseField",
]
    


class MixField(object):
    """
    docstring
    """

    def __init__(self):
        pass
    def __call__(self, value):
        return value


class MixObjectID(MixField):

    def __call__(self, value):
        return ObjectId(value)

class MixRef(MixField):

    def __init__(self, document_type=None):
        self.document_type = document_type
    
    def _get_ref_obj(self, key):
        id_field = self.document_type._meta['id_field']
        obj = self.document_type.objects(**{id_field:key}).first()
        return obj

    def __call__(self, key):
        return self._get_ref_obj(key)

class MixList(MixRef):
    
    def __init__(self, field=None):
        self.field = field
        self.document_type = None
        if field is not None:
            self.document_type = field.document_type
    def __call__(self, value):
        if self.document_type is None:
            return value
        else:
            obj = self._get_ref_obj(value)
            return obj
        return value

class MixDict(MixField):
    """
    docstring
    """
    def __call__(self, value):
        if isinstance(value, str):
            return json.loads(value)
        return value



class MixDate(MixField):

    def __call__(self, value):
        return datetime.date.fromisoformat(value)

class MixDateTime(MixField):

    def __call__(self, value):
        return datetime.datetime.fromisoformat(value)

def type_trans(field):
    pass

def parser_field(name, field, **kws):
    if isinstance(field, (StringField, EmailField, URLField)):
        return Argument(name, type=str, **kws)
    if isinstance(field, (ObjectId, ObjectIdField)):
        return Argument(name, type=MixObjectID(), **kws)
    if isinstance(field, (IntField, LongField)):
        return Argument(name, type=int, **kws)
    if isinstance(field, (BooleanField,)):
        return Argument(name, type=bool, **kws)
    if isinstance(field, (FloatField, DecimalField)):
        return Argument(name, type=float, **kws)
    if isinstance(field, (ListField,)):
        return Argument(name, type=MixList(field.field), action='append', **kws)
    if isinstance(field, (DictField, PointField, GeoJsonBaseField)):
        return Argument(name, type=MixDict(), **kws)
    if isinstance(field, ReferenceField):
        return Argument(name, type=MixRef(field.document_type), **kws)
    if isinstance(field, DynamicField):
        return Argument(name, type=MixField(), action='append', **kws)
    if isinstance(field, DateField):
        return Argument(name, type=MixDate(), **kws)
    if isinstance(field, DateTimeField):
        return Argument(name, type=MixDateTime(), **kws)
    return Argument(name, type=str)


from setuptools import Extension, dist, find_packages, setup

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()
# python setup.py sdist bdist_wheel
# python -m twine upload --repository flask-rest-mongo dist/*
setup(
    name='flask-rest-mongo',
    version='0.1.4',
    description='Helper utils for mongoDB in flask use flask-rest',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/w-copper/Flask-rest-mongo.git",
    keywords='flask,mongoDB,restful',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Utilities',
    ],
    author='Wang Tong',
    author_email='copper.w@foxmail.com',
    install_requires=[
        'Flask',
        'flask-mongoengine',
        'mongoengine',
        'flask-restful'
    ],
    python_requires='>=3.7',
    zip_safe=False
)